#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
//
// Verify STR#1895 right aligned resizable box child -- erco 7/15/14
// Build with: fltk-config  --use-images --compile str1895_test_erco_v1.cxx
// Use +/- to enlarge/shring "tux" box, should stay right aligned
// properly when vertical scrollbar appears (without being obscured,
// and without overcompensating with an offset)
//
class MyScroll : public Fl_Scroll {
    Fl_Box *mybox;
    float scale;

    // Manage child repositioning..
    void repo() {
        // First move the box to the upper right *away* from vscrollbar.
	// (don't want child too far right, or it might trigger hscroll
	// which might in turn might cause bottom of child to hit it,
	// and trigger the vscroll..!)
	// 
	Fl_Boxtype b = box();
	Fl_Scroll::ScrollInfo::Fl_Region_XYWH foobar;
	int topright_x = x()+Fl::box_dx(b) + w()-Fl::box_dw(b);	// top right x of scroll area
	int topright_y = y()+Fl::box_dy(b);			// top right y of scroll area
	mybox->position(topright_x - scrollbar.w() - mybox->w(), topright_y);  // temp move child

	// Now that child is moved, recalc scrollbars to see if e.g.
	// the current h() of the box would trigger vscrollbar to appear.
	ScrollInfo si;
	recalc_scrollbars(si);

	// If no vscrollbar, move box to *far* right
	if ( !si.vneeded )
	    mybox->position(topright_x - mybox->w(), topright_y);

	redraw();
    }

    void resize(int X,int Y,int W,int H) {

        // First resize the scroll..
        Fl_Scroll::resize(X,Y,W,H);

	// Then recalc position of child
	repo();
    }


public:
    MyScroll(int X, int Y, int W, int H, const char*L=0) : Fl_Scroll(X,Y,W,H,L) {  
	box(FL_BORDER_BOX);
	color(50);
        scale = 1.0;
        begin();
        mybox = new Fl_Box(0,0,350,386,"tux");		// make a box of some size
	mybox->align(FL_ALIGN_INSIDE|FL_ALIGN_CENTER);
	mybox->box(FL_BORDER_BOX);
	mybox->color(10);
        end();
	repo();
    }
    static void Smaller_CB(Fl_Widget *w, void *data) {
        MyScroll *scroll = (MyScroll*)data;
	int W = scroll->mybox->w();
	int H = scroll->mybox->h();
	W *= 0.9;
	H *= 0.9;
	scroll->mybox->size(W,H);
	scroll->repo();
    }
    static void Larger_CB(Fl_Widget *w, void *data) {
        MyScroll *scroll = (MyScroll*)data;
	int W = scroll->mybox->w();
	int H = scroll->mybox->h();
	W *= 1.1;
	H *= 1.1;
	scroll->mybox->size(W,H);
	scroll->repo();
    }
};

int main() {
    Fl_Double_Window win(720,486);
    MyScroll *scroll = new MyScroll(10,10,720-20,486-60);

    int y = scroll->y() + scroll->h();
    Fl_Button smaller(10, y+10, 25,25, "-");
    smaller.callback(MyScroll::Smaller_CB, (void*)scroll);

    Fl_Button bigger(40, y+10, 25,25, "+");
    bigger.callback(MyScroll::Larger_CB, (void*)scroll);

    win.resizable(win);
    win.show();
    return(Fl::run());
}
